module Pagseguro
  module Charge
    class ChargeResponse

      attr_accessor :reference_id,
                    :id,
                    :status,
                    :created_at,
                    :description,
                    :amount,
                    :payment_response,
                    :payment_method,
                    :error_messages,
                    :links

      STATUS_AUTHORIZED = 'AUTHORIZED'.freeze
      STATUS_DECLINED = 'DECLINED'.freeze
      STATUS_PAID = 'PAID'.freeze
      STATUS_CANCELED = 'CANCELED'.freeze
      STATUS_PENDING = 'WAITING'.freeze

      def initialize
      end

      def to_json(*options)
        hash = as_json(*options)
        hash.reject! {|k,v| v.nil?}
        hash.to_json(*options)
      end

      def self.fill_from_json(data)
        return if data.nil?

        charge_response = new

        charge_response.reference_id = data["reference_id"]
        charge_response.id = data["id"]
        charge_response.status = data["status"]
        charge_response.amount = Amount.fill_from_json(data["amount"])
        charge_response.created_at = data["created_at"]
        charge_response.description = data["description"]
        charge_response.payment_response = PaymentResponse.fill_from_json(data["payment_response"])
        charge_response.payment_method = PaymentMethod.fill_from_json(data["payment_method"])

        charge_response.error_messages = data["error_messages"].map{|a| ErrorMessage.fill_from_json(a)} unless data["error_messages"].blank?
        charge_response.links = data["links"].map{|a| Link.fill_from_json(a)} unless data["links"].blank?

        charge_response
      end


      def as_json(options={})
        {
          id: @id,
          reference_id: @reference_id,
          status: @status,
          amount: @amount,
          created_at: @created_at,
          description: @description,
          payment_response: @payment_response,
          payment_method: @payment_method,
          error_messages: @error_messages,
          links: @links
        }
      end

      def paid?
        @status == STATUS_PAID
      end
      def authorized?
        @status == STATUS_AUTHORIZED
      end

      def declined?
        @status == STATUS_DECLINED
      end

      def captured?
        @status == STATUS_PAID
      end

      def canceled?
        @status == STATUS_CANCELED
      end
    end



  end
end
