module Pagseguro
  module Charge
    module Connect
      class ChargeConnect
        PRODUCTION_URL = 'https://api.pagseguro.com/charges'
        SANDBOX_URL = 'https://sandbox.api.pagseguro.com/charges'

        attr_accessor :authorization_token, :sandbox
        private :authorization_token

        def initialize(authorization_token, sandbox = false)
          @authorization_token = authorization_token
          @sandbox = sandbox
        end


        # creates a new charge request
        def place_charge(charge_request)
          data = do_connect('POST', URI.parse(@sandbox ? SANDBOX_URL : PRODUCTION_URL), charge_request.to_json)
          charge_response = ChargeResponse.fill_from_json(JSON.parse(data))
          charge_response
        end

        # retireves a charge by its id
        def retrieve_by_id(charge_id)
          data = do_connect('GET', URI.parse("#{@sandbox ? SANDBOX_URL : PRODUCTION_URL}/#{charge_id}"))
          charge_response = ChargeResponse.fill_from_json(JSON.parse(data))
          charge_response
        end

        # retireves a charge by reference number
        def retrieve_by_reference_id(reference_id)
          data = do_connect('GET', URI.parse("#{@sandbox ? SANDBOX_URL : PRODUCTION_URL}?reference_id=#{reference_id}"))
          charge_response = ChargeResponse.fill_from_json(JSON.parse(data))
          charge_response
        end

        # captures an authorized payment
        def capture(charge_request)
          data = do_connect('POST', URI.parse("#{@sandbox ? SANDBOX_URL : PRODUCTION_URL}/#{charge_request.id}/capture"),
                            charge_request.to_json)
          charge_response = ChargeResponse.fill_from_json(JSON.parse(data))
          charge_response
        end

        #void/cancels a payment
        def cancel(charge_request)
          data = do_connect('POST', URI.parse("#{@sandbox ? SANDBOX_URL : PRODUCTION_URL}/#{charge_request.id}/cancel"),
                            charge_request.to_json)
          charge_response = ChargeResponse.fill_from_json(JSON.parse(data))
          charge_response
        end


        private
        def do_connect(method, uri, data=nil)

          http = Net::HTTP.new(uri.host, uri.port)
          http.use_ssl = true

          request = case method
          when 'POST'
            Net::HTTP::Post.new(uri)
          when 'GET'
            Net::HTTP::Get.new(uri)
          else
            Net::HTTP::Post.new(uri)
          end

          request["X-api-version"] = "1.0"
          request["Accept"] = "application/json"
          request["Cache-Control"] = "no-cache"
          request["Authorization"] = "Bearer #{@authorization_token}"

          if data.nil?
            request["Content-Length"] = "0"
          else
            request.content_type = "application/json"
            request.body = data
          end

          puts "REQUEST #{request["Authorization"]}"
          puts "REQUEST BODY #{request.body}"
          response = http.request(request)

          # raise Error.new(data.first["Code"], data.first["Message"]) if response.code.to_i >= 400

          puts "RESPONSE BODY #{response.body}"
          response.body
        end
      end

    end
  end
end