module Pagseguro
  module Charge
    # Payment data
    #
    # @attr [String] payment_method States the payment method used in the charge. For now, we only support credit cards (“credit_card”) and boleto ("boleto")
    # @attr [Object] card The Credit Card object
    # @attr [Integer] amount Gross amount of the charge, in cents.
    # @attr [Integer] installments The number of monthly installments the card will be charged. This must be an one or two-digit integer between 1 and 12.
    # @attr [Boolean] capture Represents if the charge must be created and captured. For now, ‘TRUE’ is the only supported value.
    class PaymentMethod

      attr_accessor :type,
                    :card,
                    :installments,
                    :capture,
                    :capture_before,
                    :boleto


      def initialize(type = Pagseguro::Charge::ChargeRequest::TYPE_CREDITCARD)
        @type = type
      end

      def to_json(*options)
        hash = as_json(*options)
        hash.reject! {|k,v| v.nil?}
        hash.to_json(*options)
      end

      def self.fill_from_json(data)
        return if data.nil?

        payment_method = new

        payment_method.type = data["type"]
        payment_method.card = CreditCard.fill_from_json(data["card"])
        payment_method.boleto = Boleto.fill_from_json(data["boleto"])
        payment_method.installments = data["installments"]
        payment_method.capture = data["capture"]
        payment_method.capture_before = data["capture_before"]
        payment_method
      end


      def as_json(options={})
        {
          type: @type,
          installments: @installments,
          capture: @capture,
          capture_before: @capture_before,
          card: @card,
          boleto: @boleto,
        }
      end
    end


  end
end
